package com.iformall.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.iformall.domain.po.TtMerchantPoi;
import com.iformall.domain.po.WxAppinfo;
import com.iformall.enums.EnumAppPlat;
import com.iformall.enums.EnumAppType;
import com.iformall.mapper.TtMerchantPoiMapper;
import com.iformall.mapper.WxAppinfoMapper;
import com.iformall.service.TtMerchantPoiService;
import com.iformall.service.toutiao.FmTtOpenService;
import com.iformall.service.toutiao.api.TtOpenMaService;
import com.iformall.service.toutiao.api.bean.TtOpenResult;
import com.iformall.service.toutiao.api.bean.TtOpenShopMaterial;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;


/**
 * @author gongbiao
 */
@Service
public class TtMerchantPoiServiceImpl implements TtMerchantPoiService {

    private final Logger logger = LoggerFactory.getLogger(this.getClass());

    @Autowired
    TtMerchantPoiMapper ttMerchantPoiMapper;

    @Autowired
    WxAppinfoMapper wxAppinfoMapper;

    @Autowired
    protected FmTtOpenService openService;


    @Override
    public PageInfo<TtMerchantPoi> listAsPage(TtMerchantPoi record, Integer pageIndex, Integer pageSize) {
        return PageHelper.startPage(pageIndex, pageSize).doSelectPageInfo(() -> ttMerchantPoiMapper.findList(record));
    }

    @Override
    public TtMerchantPoi getById(Long id) {
        return ttMerchantPoiMapper.selectById(id);
    }

    @Override
    public boolean addShopMaterial(WxAppinfo appinfo, TtMerchantPoi merchantPoi) {

        WxAppinfo appinfoQ = new WxAppinfo();
        appinfoQ.setTenantId(appinfo.getTenantId());
        appinfoQ.setPlat(EnumAppPlat.TOUTIAO.getCode());
        appinfoQ.setType(EnumAppType.A.getCode());
        WxAppinfo ttopen = wxAppinfoMapper.selectOne(new QueryWrapper<>(appinfoQ));

        TtOpenShopMaterial shopMaterial = new TtOpenShopMaterial();
        shopMaterial.setClientKey(ttopen.getAppId());
        shopMaterial.setSupplierExtId(merchantPoi.getSupplierExtId());
        shopMaterial.setBizLicencePath(merchantPoi.getBizLicencePath());
        shopMaterial.setContractPath(merchantPoi.getContractPath());
        shopMaterial.setQualMaterial(merchantPoi.getQualMaterial());
        shopMaterial.setExpireTime(merchantPoi.getExpireTime());
        shopMaterial.setAdditionalQual(merchantPoi.getAdditionalQual());

        TtOpenMaService openMaService = openService.getTtOpenComponentService().getTtMaServiceByAppid(appinfo.getAppId());

        try{
            TtOpenResult res = openMaService.addShopMaterial(shopMaterial);
            if(res.isSuccess()){
                merchantPoi.setUpdateDate(new Date());
                ttMerchantPoiMapper.updateById(merchantPoi);
                return true;
            }
            logger.error("提交商铺资质材料error"+ res.getMessage());
        }catch(Exception e){
            logger.error("提交商铺资质材料error"+ e.getMessage());
        }
        return false;
    }



}
