package com.iformall.controller;

import com.iformall.common.Result;
import com.iformall.common.ResultData;
import com.iformall.service.wechat.FmOpenService;
import io.swagger.annotations.Api;
import me.chanjar.weixin.common.error.WxErrorException;
import me.chanjar.weixin.open.bean.WxOpenMaCodeTemplate;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * Stormeye Wu
 */
@RestController
@RequestMapping("/weappDraftTemplate")
@Api(description = "微信第三方开发平台-小程序-代码模版库管理")
public class WechatWeappDraftTemplateController {
    private final Logger logger = LoggerFactory.getLogger(getClass());
    @Autowired
    private FmOpenService openService;


    @GetMapping("/tempDraftList")
    public ResultData getTempDraftList() {
        try {
            List<WxOpenMaCodeTemplate> list =  openService.getWxOpenComponentService().getTemplateDraftList();
            return new ResultData(list);
        } catch (WxErrorException e) {
            logger.error(e.getMessage());
            return new ResultData(Result.ERROR, e.getMessage());
        }

    }

    @GetMapping("/tempList")
    public ResultData getTempList() {
        try {
            List<WxOpenMaCodeTemplate> list =  openService.getWxOpenComponentService().getTemplateList();
            return new ResultData(list);
        } catch (WxErrorException e) {
            logger.error(e.getMessage());
            return new ResultData(Result.ERROR, e.getMessage());
        }
    }

    @PostMapping("/addToTemplate")
    public ResultData addToTemplate(@RequestBody WxOpenMaCodeTemplate template) {
        if(template == null || template.getDraftId() == null) {
            return new ResultData(Result.ERROR, "draftId is null");
        }
        try {
            openService.getWxOpenComponentService().addToTemplate(template.getDraftId());
            return new ResultData();
        } catch (WxErrorException e) {
            logger.error(e.getMessage());
            return new ResultData(Result.ERROR, e.getMessage());
        }
    }

    @PostMapping("/deleteTemplate")
    public ResultData deleteTemplate(@RequestBody WxOpenMaCodeTemplate template) {
        if(template == null || template.getTemplateId() == null) {
            return new ResultData(Result.ERROR, "templateId is null");
        }
        try {
            openService.getWxOpenComponentService().deleteTemplate(template.getTemplateId());
            return new ResultData();
        } catch (WxErrorException e) {
            logger.error(e.getMessage());
            return new ResultData(Result.ERROR, e.getMessage());
        }
    }
}
