package com.iformall.controller.card;

import com.iformall.common.ResultData;
import com.iformall.controller.base.BaseController;
import com.iformall.service.wechat.FmOpenService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import me.chanjar.weixin.common.api.WxConsts;
import me.chanjar.weixin.common.error.WxErrorException;
import me.chanjar.weixin.mp.api.WxMpService;
import me.chanjar.weixin.mp.bean.card.*;
import me.chanjar.weixin.mp.bean.kefu.WxMpKefuMessage;
import me.chanjar.weixin.mp.bean.membercard.WxMpMemberCardCreateMessage;
import me.chanjar.weixin.mp.bean.membercard.WxMpMemberCardUpdateMessage;
import me.chanjar.weixin.mp.bean.membercard.WxMpMemberCardUpdateResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.List;

@RestController
@RequestMapping("wxCardIO")
@Api(description = "微信卡券接口")
public class WxCardController extends BaseController {
    private final Logger logger = LoggerFactory.getLogger(this.getClass());

    @Autowired
    private FmOpenService openService;

    @ApiOperation("创建卡券")
    @GetMapping("createCard/{appId}")
    public ResultData createCard(@PathVariable String appId, String type) {
        logger.debug("[" + getIpAddr() + "] WxCardController::createCard");

        BaseInfo base = new BaseInfo();
        base.setLogoUrl("http://mmbiz.qpic.cn/mmbiz_png/2kbqxXJ9VM94Uq1YAsZDPKcWnNIOU4lv2PJmgIf1HIicldDibj2xcXjwSeqI7ZTfUj0wOokFmurO6gA7ppzF0pug/0?wx_fmt=png");
        base.setBrandName("测试优惠券");
        base.setCodeType("CODE_TYPE_QRCODE");
        base.setTitle("测试标题");
        base.setColor("Color010");
        base.setNotice("测试Notice");
        base.setServicePhone("020-88888888");
        base.setDescription("不可与其他优惠同享\\n如需团购券发票，请在消费时向商户提出\\n店内均可使用，仅限堂食");
        DateInfo info = new DateInfo();
        info.setType("DATE_TYPE_FIX_TERM");
        info.setFixedBeginTerm(0);
        info.setFixedTerm(30);
        base.setDateInfo(info);
        Sku sku = new Sku();
        sku.setQuantity(100);
        base.setSku(sku);
        base.setGetLimit(1);
        base.setCanShare(true);
        base.setCanGiveFriend(true);
        base.setUseAllLocations(true);
        base.setCenterTitle("顶部居中按钮");
        base.setCenterSubTitle("按钮下方的wording");
        base.setCenterUrl("www.qq.com");
        base.setCustomUrl("http://www.qq.com");
        base.setCustomUrlName("立即使用");
        base.setCustomUrlSubTitle("副标题tip");
        base.setPromotionUrlName("更多优惠");
        base.setPromotionUrl("http://www.qq.com");
        //TODO base.setLocationIdList("1234");
        WxMpService mpService = openService.getWxOpenComponentService().getWxMpServiceByAppid(appId);

        if (type.equalsIgnoreCase("groupon")) {
            //团购券
            WxMpCardCreateRequest grouponMessage = new WxMpCardCreateRequest();
            GrouponCardCreateRequest grouponCardCreateRequest = new GrouponCardCreateRequest();
            GrouponCard grouponCard = new GrouponCard();
            grouponCard.setBaseInfo(base);
            grouponCard.setDealDetail("deal detail");

            grouponCardCreateRequest.setGroupon(grouponCard);
            grouponMessage.setCardCreateRequest(grouponCardCreateRequest);

            try {

                WxMpCardCreateResult result = mpService.getCardService().createCard(grouponMessage);
                logger.info(result.toString());
            } catch (WxErrorException e) {
                logger.error(e.getMessage());
            }
        } else if (type.equalsIgnoreCase("cash")) {
            //现金券
            WxMpCardCreateRequest cashMessage = new WxMpCardCreateRequest();
            CashCardCreateRequest cashCardCreateRequest = new CashCardCreateRequest();
            CashCard cashCard = new CashCard();
            cashCard.setBaseInfo(base);
            cashCard.setLeastCost(1000);
            cashCard.setReduceCost(100);

            cashCardCreateRequest.setCash(cashCard);
            cashMessage.setCardCreateRequest(cashCardCreateRequest);
            try {
                WxMpCardCreateResult result = mpService.getCardService().createCard(cashMessage);
                logger.info(result.toString());
            } catch (WxErrorException e) {
                logger.error(e.getMessage());
            }
        } else if (type.equalsIgnoreCase("discount")) {
            //折扣券
            WxMpCardCreateRequest discountMessage = new WxMpCardCreateRequest();
            DiscountCardCreateRequest discountCardCreateRequest = new DiscountCardCreateRequest();
            DiscountCard discountCard = new DiscountCard();
            discountCard.setBaseInfo(base);
            discountCard.setDiscount(30);

            discountCardCreateRequest.setDiscount(discountCard);
            discountMessage.setCardCreateRequest(discountCardCreateRequest);

            try {
                WxMpCardCreateResult result = mpService.getCardService().createCard(discountMessage);
                logger.info(result.toString());
            } catch (WxErrorException e) {
                logger.error(e.getMessage());
            }
        } else if (type.equalsIgnoreCase("gift")) {
            //兑换券
            WxMpCardCreateRequest giftMessage = new WxMpCardCreateRequest();
            GiftCardCreateRequest giftCardCreateRequest = new GiftCardCreateRequest();
            GiftCard giftCard = new GiftCard();
            giftCard.setBaseInfo(base);
            giftCard.setGift("星巴克雪瑞纳咖啡大杯");

            giftCardCreateRequest.setGift(giftCard);
            giftMessage.setCardCreateRequest(giftCardCreateRequest);
            try {
                WxMpCardCreateResult result = mpService.getCardService().createCard(giftMessage);
                logger.info(result.toString());
            } catch (WxErrorException e) {
                logger.error(e.getMessage());
            }
        } else if (type.equalsIgnoreCase("generalcard")) {
            //普通兑换券
            WxMpCardCreateRequest generalMessage = new WxMpCardCreateRequest();
            GeneralCouponCreateRequest generalCardCreateRequest = new GeneralCouponCreateRequest();
            GeneralCoupon generalCard = new GeneralCoupon();
            generalCard.setBaseInfo(base);
            generalCard.setDefaultDetail("音乐木盒");

            generalCardCreateRequest.setGeneralCoupon(generalCard);
            generalMessage.setCardCreateRequest(generalCardCreateRequest);
            try {
                WxMpCardCreateResult result = mpService.getCardService().createCard(generalMessage);
                logger.info(result.toString());
            } catch (WxErrorException e) {
                logger.error(e.getMessage());
            }
        } else if (type.equalsIgnoreCase("membercard")) {
            //会员卡
            WxMpMemberCardCreateMessage createMessage = new WxMpMemberCardCreateMessage();
            MemberCardCreateRequest cardCreateRequest = new MemberCardCreateRequest();
            MemberCard memberCard = new MemberCard();
            memberCard.setPrerogative("特权说明");
            //激活方式
            memberCard.setAutoActivate(true);//自动激活
//    memberCard.setActivateUrl("http://www.qq.com");
//    memberCard.setWxActivate(false);//微信激活
            memberCard.setSupplyBonus(true);
            memberCard.setSupplyBalance(false);
            memberCard.setBackgroundPicUrl(null);
            memberCard.setDiscount(0);

            BaseInfo baseInfo = new BaseInfo();
            baseInfo.setLogoUrl("http://mmbiz.qpic.cn/mmbiz_png/2kbqxXJ9VM94Uq1YAsZDPKcWnNIOU4lv2PJmgIf1HIicldDibj2xcXjwSeqI7ZTfUj0wOokFmurO6gA7ppzF0pug/0?wx_fmt=png");
            baseInfo.setCodeType("CODE_TYPE_QRCODE");
            baseInfo.setBrandName("金泸科技");
            baseInfo.setTitle("铂金用户贵宾卡");
            baseInfo.setColor("Color010");
            baseInfo.setNotice("卡券使用提醒");
            baseInfo.setDescription("卡券使用说明");
            baseInfo.setServicePhone("4008803016");
            //商品信息
            Sku sku1 = new Sku();
            baseInfo.setSku(sku1);
            //使用日期
            DateInfo dateInfo = new DateInfo();
            baseInfo.setDateInfo(dateInfo);
            PayInfo payInfo = new PayInfo();
            SwipeCard swipeCard = new SwipeCard();
            swipeCard.setSwipeCard(true);
            payInfo.setSwipeCard(swipeCard);
            baseInfo.setPayInfo(payInfo);

            memberCard.setBaseInfo(baseInfo);
            memberCard.setBonusRules("请参考积分规则");

            cardCreateRequest.setMemberCard(memberCard);
            createMessage.setCardCreateRequest(cardCreateRequest);

            try {
                WxMpCardCreateResult result = mpService.getMemberCardService().createMemberCard(createMessage);
                logger.info(result.toString());
            } catch (WxErrorException e) {
                logger.error(e.getMessage());
            }
        } else if (type.equalsIgnoreCase("giftcard")) {
            //礼品卡
            WxMpMemberCardCreateMessage createMessage = new WxMpMemberCardCreateMessage();
            MemberCardCreateRequest cardCreateRequest = new MemberCardCreateRequest();
            MemberCard memberCard = new MemberCard();
            memberCard.setPrerogative("特权说明");
            //激活方式
            memberCard.setAutoActivate(true);//自动激活
//    memberCard.setActivateUrl("http://www.qq.com");
//    memberCard.setWxActivate(false);//微信激活
            memberCard.setSupplyBonus(false);
            memberCard.setSupplyBalance(false);
            memberCard.setBackgroundPicUrl(null);
            memberCard.setDiscount(0);

            BaseInfo baseInfo = new BaseInfo();
            baseInfo.setLogoUrl("http://mmbiz.qpic.cn/mmbiz_png/2kbqxXJ9VM94Uq1YAsZDPKcWnNIOU4lv2PJmgIf1HIicldDibj2xcXjwSeqI7ZTfUj0wOokFmurO6gA7ppzF0pug/0?wx_fmt=png");
            baseInfo.setCodeType("CODE_TYPE_QRCODE");
            baseInfo.setBrandName("金泸科技");
            baseInfo.setTitle("铂金用户贵宾卡");
            baseInfo.setColor("Color010");
            baseInfo.setNotice("卡券使用提醒");
            baseInfo.setDescription("卡券使用说明");
            baseInfo.setServicePhone("4008803016");
            //商品信息
            Sku sku1 = new Sku();
            baseInfo.setSku(sku1);
            //使用日期
            DateInfo dateInfo = new DateInfo();
            baseInfo.setDateInfo(dateInfo);

            memberCard.setBaseInfo(baseInfo);
            memberCard.setBonusRules("请参考积分规则");

            cardCreateRequest.setMemberCard(memberCard);
            createMessage.setCardCreateRequest(cardCreateRequest);

            try {
                WxMpCardCreateResult result = mpService.getMemberCardService().createMemberCard(createMessage);
                logger.info(result.toString());
            } catch (WxErrorException e) {
                logger.error(e.getMessage());
            }
        }
        return new ResultData();
    }


    @ApiOperation("发送客服消息-用户卡券")
    @GetMapping("sendCard/{appId}")
    public ResultData sendCard(@PathVariable String appId, String cardId) {
        logger.debug("[" + getIpAddr() + "] WxCardController::sendCard");
        WxMpKefuMessage kfMessage = new WxMpKefuMessage();
        String openId = "onxhD5tpknKjLCxMe0FKUULE8zUg";
        kfMessage.setToUser(openId);
        kfMessage.setMsgType(WxConsts.KefuMsgType.WXCARD);
        kfMessage.setCardId(cardId);

        try {
            WxMpService mpService = openService.getWxOpenComponentService().getWxMpServiceByAppid(appId);
            boolean result = mpService.getKefuService().sendKefuMessage(kfMessage);
            logger.info("{}", result);
        } catch (WxErrorException e) {
            logger.error(e.getMessage());
        }
        return new ResultData();
    }

    @ApiOperation("获取用户卡券")
    @GetMapping("getCardList/{appId}")
    public ResultData getCardList(@PathVariable String appId, String openId) {
        logger.debug("[" + getIpAddr() + "] WxCardController::getCardList");
//        try {
//            WxMpService mpService = openService.getWxOpenComponentService().getWxMpServiceByAppid(appId);
//            WxUserCardListResult result = mpService.getCardService().getUserCardList(openId, null);
//            logger.info(result.toString());
//        } catch (WxErrorException e) {
//            logger.error(e.getMessage());
//        }
        try {
            WxMpService mpService = openService.getWxOpenComponentService().getWxMpServiceByAppid(appId);
            WxUserCardListResult result = mpService.getCardService().getUserCardList(openId, null);
            logger.info(result.toString());
        } catch (WxErrorException e) {
            logger.error(e.getMessage());
        }
        return new ResultData();
    }

    @ApiOperation("查询Code接口")
    @GetMapping("cardCodeGet/{appId}")
    public ResultData cardCodeGet(@PathVariable String appId, String code) {
        logger.debug("[" + getIpAddr() + "] WxCardController::cardCodeGet");
        try {
            WxMpService mpService = openService.getWxOpenComponentService().getWxMpServiceByAppid(appId);
            WxMpCardResult result = mpService.getCardService().queryCardCode(null, code, true);
            logger.info(result.toString());
        } catch (WxErrorException e) {
            logger.error(e.getMessage());
        }
        return new ResultData();
    }

    @ApiOperation("核销Code接口")
    @GetMapping("consumeCode/{appId}")
    public ResultData consumeCode(@PathVariable String appId, String code) {
        logger.debug("[" + getIpAddr() + "] WxCardController::consumeCode");
        try {
            WxMpService mpService = openService.getWxOpenComponentService().getWxMpServiceByAppid(appId);
            String result = mpService.getCardService().consumeCardCode(code, null);
            logger.info(result);
        } catch (WxErrorException e) {
            logger.error(e.getMessage());
        }
        return new ResultData();
    }

    @ApiOperation("查看卡券详情")
    @GetMapping("cardGet/{appId}")
    public ResultData cardGet(@PathVariable String appId, String cardId) {
        logger.debug("[" + getIpAddr() + "] WxCardController::cardGet");
        try {
            WxMpService mpService = openService.getWxOpenComponentService().getWxMpServiceByAppid(appId);
            String result = mpService.getCardService().getCardDetail(cardId);
            logger.info(result);
        } catch (WxErrorException e) {
            logger.error(e.getMessage());
        }
        return new ResultData();
    }

    @ApiOperation(value = "批量查询卡券列表", notes = "status: CARD_STATUS_NOT_VERIFY”, 待审核 ； “CARD_STATUS_VERIFY_FAIL”, 审核失败； “CARD_STATUS_VERIFY_OK”， 通过审核； “CARD_STATUS_DELETE”， 卡券被商户删除； “CARD_STATUS_DISPATCH”,在公众平台投放过的卡券")
    @GetMapping("cardBatchGet/{appId}")
    public ResultData cardBatchGet(@PathVariable String appId, String status) {
        logger.debug("[" + getIpAddr() + "] WxCardController::cardBatchGet");
        List<String> statusList = new ArrayList<>();
        statusList.add(status);
        try {
            WxMpService mpService = openService.getWxOpenComponentService().getWxMpServiceByAppid(appId);
            WxMpCardBatchGetResult result = mpService.getCardService().getCardBatchGet(0, 10, statusList);
            logger.info(result.toString());
        } catch (WxErrorException e) {
            logger.error(e.getMessage());
        }
        return new ResultData();
    }

    @ApiOperation(value = "更新卡信息")
    @GetMapping("updateCardUser/{appId}")
    public ResultData updateCardUser(@PathVariable String appId, String code) {
        logger.debug("[" + getIpAddr() + "] WxCardController::updateCardUser");
        String cardId = "pnxhD5rozwYIX7j210S9thySoN0w";
        WxMpMemberCardUpdateMessage updateMessage = new WxMpMemberCardUpdateMessage();
        //TODO updateMessage.setAddBounus(100);
        updateMessage.setBonus(1000);
        updateMessage.setCardId(cardId);
        updateMessage.setCode(code);

        WxMpService mpService = openService.getWxOpenComponentService().getWxMpServiceByAppid(appId);
        try {
            WxMpMemberCardUpdateResult result = mpService.getMemberCardService().updateUserMemberCard(updateMessage);
            logger.info(result.toString());
        } catch (WxErrorException e) {
            logger.error(e.getMessage());
        }
        return new ResultData();
    }

    @ApiOperation(value = "失效卡")
    @GetMapping("unavailableCardCode/{appId}")
    public ResultData unavailableCardCode(@PathVariable String appId, String code) {
        logger.debug("[" + getIpAddr() + "] WxCardController::unavailableCardCode");
        String cardId = "pnxhD5rozwYIX7j210S9thySoN0w";
        String reason = "会员卡退卡";

        WxMpService mpService = openService.getWxOpenComponentService().getWxMpServiceByAppid(appId);
        try {
            String result = mpService.getCardService().unavailableCardCode(cardId, code, reason);
            logger.info(result);
        } catch (WxErrorException e) {
            logger.error(e.getMessage());
        }
        return new ResultData();
    }
}
